ArkInventory.LDB = {
	Bags = ArkInventory.Lib.DataBroker:NewDataObject( string.format( "%s_%s", ArkInventory.Const.Program.Name, ArkInventory.Const.Slot.Data[ArkInventory.Const.Location.Bag].name ), {
		type = "data source",
		icon = ArkInventory.Global.Location[ArkInventory.Const.Location.Bag].Texture,
		text = ArkInventory.Const.Slot.Data[ArkInventory.Const.Location.Bag].name,
	} ),
	Money = ArkInventory.Lib.DataBroker:NewDataObject( string.format( "%s_%s", ArkInventory.Const.Program.Name, MONEY ), {
		type = "data source",
		icon = [[Interface\Icons\INV_Misc_Coin_02]],
		text = "Money",
	} ),
	Ammo = ArkInventory.Lib.DataBroker:NewDataObject( string.format( "%s_%s", ArkInventory.Const.Program.Name, "Ammo" ), {
		type = "data source",
		icon = [[Interface\Icons\INV_Misc_Ammo_Bullet_02]],
		text = "Ammo",
	} ),
	Companion = { },
	Pets = ArkInventory.Lib.DataBroker:NewDataObject( string.format( "%s_%s", ArkInventory.Const.Program.Name, ArkInventory.Localise["WOW_ITEM_TYPE_MISC_PET"] ), {
		type = "data source",
		icon = ArkInventory.Global.Location[ArkInventory.Const.Location.Pet].Texture,
		text = "",
	} ),
	Mounts = ArkInventory.Lib.DataBroker:NewDataObject( string.format( "%s_%s", ArkInventory.Const.Program.Name, ArkInventory.Localise["WOW_ITEM_TYPE_MISC_MOUNT"] ), {
		type = "data source",
		icon = ArkInventory.Global.Location[ArkInventory.Const.Location.Mount].Texture,
		text = "",
		types = { "flying", "ground", "water" },
	} ),
	Tracking_Currency = ArkInventory.Lib.DataBroker:NewDataObject( string.format( "%s_%s_%s", ArkInventory.Const.Program.Name, "Tracking", "Currency" ), {
		type = "data source",
		icon = ArkInventory.Global.Location[ArkInventory.Const.Location.Token].Texture,
		text = "",
	} ),
	Tracking_Item = ArkInventory.Lib.DataBroker:NewDataObject( string.format( "%s_%s_%s", ArkInventory.Const.Program.Name, "Tracking", "Item" ), {
		type = "data source",
		icon = [[Interface\Icons\Ability_Tracking]],
		text = "",
	} ),
}

local companionTable = { }



function ArkInventory.LDB.Bags:OnClick( button )
	if button == "RightButton" then
		ArkInventory.MenuLDBBagsOpen( self )
	else
		ArkInventory.Frame_Main_Toggle( ArkInventory.Const.Location.Bag )
	end
end

function ArkInventory.LDB.Bags:Update( )
	local loc_id = ArkInventory.Const.Location.Bag
	--ArkInventory.LDB.Bags.icon = ArkInventory.Global.Location[loc_id].Texture
	self.text = ArkInventory.Frame_Status_Update_Empty( loc_id, ArkInventory.Global.Me, true )
end



function ArkInventory.LDB.Money:Update( )
	ArkInventory.LDB.Money.text = ArkInventory.MoneyText( GetMoney( ) )
end

function ArkInventory.LDB.Money.OnTooltipShow( frame )
	ArkInventory.MoneyFrame_Tooltip( frame )
end



function ArkInventory.LDB.Ammo:OnClick( button )
	
	if ArkInventory.IsCataclysm( ) then return end
	
	if button == "RightButton" then
		ArkInventory.MenuLDBAmmoOpen( self )
	end
	
end

function ArkInventory.LDB.Ammo:Update( )
	
	if ArkInventory.IsCataclysm( ) then return end
	
	local inv_id, texture = GetInventorySlotInfo( "RangedSlot" )
	local hw = GetInventoryItemLink( "player", inv_id )
	
	if not hw then
		
		-- no ranged weapon equipped
		self.icon = texture
		self.text = ""
		
	else
		
		local d1, d2 = GetInventoryItemDurability( inv_id )
		if d1 == d2 then
			d1 = ""
		elseif d1 == 0 then
			d1 = RED_FONT_COLOR_CODE .. ArkInventory.Localise["LDB_AMMO_WEAPON_BROKEN"] .. FONT_COLOR_CODE_CLOSE
		else
			if ArkInventory.db.char.option.ldb.ammo.durability then
				d1 = string.format( "%i%%", ( d1 / d2 ) * 100 )
			else
				d1 = ""
			end
		end
		
		local needsammo = false
		
		inv_id = GetInventorySlotInfo( "AmmoSlot" )
		
		local check = { 264, 5011, 266 }
		for _, v in pairs( check ) do
			local wt = GetSpellInfo( v )
			if IsEquippedItemType( wt ) then --if select( 8, ArkInventory.ObjectInfo( hw ) ) == wt then
				needsammo = true
				break
			end
		end
		
		if not needsammo then
			
			self.icon = GetItemIcon( hw )
			self.text = d1
			
		else
			
			local na = RED_FONT_COLOR_CODE .. SPELL_FAILED_NO_AMMO .. FONT_COLOR_CODE_CLOSE
			
			local ha = GetInventoryItemLink( "player", inv_id )
			
			if not ha then
				
				-- use weapon icon
				self.icon = GetItemIcon( hw )
				
				-- show durability
				if d1 == "" then
					self.text = na
				else
					self.text = string.format( "%s (%s)", na, d1 )
				end
				
			else
				
				-- use ammo icon
				self.icon = GetItemIcon( ha )
				
				-- get ammo count
				local c = GetItemCount( ha )
				if c == 0 then c = na end
				
				-- show count and durability
				if d1 == "" then
					self.text = string.format( "%s", c )
				else
					self.text = string.format( "%s (%s)", c, d1 )
				end
				
			end
			
		end
		
	end
	
end

function ArkInventory.LDB.Ammo:OnTooltipShow( )
	
	if ArkInventory.IsCataclysm( ) then
		
		self:AddLine( "Ammunition has been removed from Cataclysm, please disable this object in your LDB display mod" )
		
	else
		
		local inv_id = GetInventorySlotInfo( "RangedSlot" )
		local h = GetInventoryItemLink( "player", inv_id )
		
		if not h then
			self:AddLine( ArkInventory.Localise["LDB_AMMO_WEAPON_NONE"] )
		else
			self:AddLine( h )
		end
		
	end

end



function ArkInventory.LDB.Tracking_Currency:Update( )
	
	if ArkInventory.IsCataclysm( ) then
		self:Update4( )
	else
		self:Update3( )
	end
	
end

function ArkInventory.LDB.Tracking_Currency:Update3( )
	
	self.text = ""
	
	-- expand all token headers
	local numTokenTypes = GetCurrencyListSize( )
	
	if numTokenTypes == 0 then return end
	
	for j = numTokenTypes, 1, -1 do
		local _, isHeader, isExpanded = GetCurrencyListInfo( j )
		if isHeader and not isExpanded then
			ExpandCurrencyList( j, 1 )
		end
	end
	
	local numTracked = 0
	numTokenTypes = GetCurrencyListSize( )
	local size = ( ArkInventory.db.char.option.ldb.tracking.currency.size or 0 ) + 1
	
	for j = 1, numTokenTypes do
		
		local name, isHeader, isExpanded, isUnused, isWatched, count, currencyType, icon, item = GetCurrencyListInfo( j )
		
		if not isHeader then
			
			if ArkInventory.db.char.option.ldb.tracking.currency.tracked[name] then
				
				if currencyType == 1 then
					icon = [[Interface\PVPFrame\PVP-ArenaPoints-Icon]]
				elseif currencyType == 2 then
					icon = [[Interface\PVPFrame\PVP-Currency-]] .. UnitFactionGroup( "player" )
				end
				
				numTracked = numTracked + 1
				size = size - 1
				self.text = string.format( "%s  |T%s:!!!:!!!|t %d", self.text, icon or ArkInventory.Const.Texture.Missing, count or 0 )
				
			end
			
		end
		
	end
	
	if numTracked > 0 then
		if size < 1 then
			size = 0
		end
		size = string.format( "%d", size )
		self.text = string.gsub( self.text, "!!!", size )
	end
	
	self.text = string.trim( self.text )
	
end

function ArkInventory.LDB.Tracking_Currency:Update4( )
	
	self.text = ""
	
	-- expand all token headers
	local numTokenTypes = GetCurrencyListSize( )
	
	if numTokenTypes == 0 then return end
	
	for j = numTokenTypes, 1, -1 do
		local _, isHeader, isExpanded = GetCurrencyListInfo( j )
		if isHeader and not isExpanded then
			ExpandCurrencyList( j, 1 )
		end
	end
	
	numTokenTypes = GetCurrencyListSize( )
	
	for j = 1, numTokenTypes do
		
		local name, isHeader, isExpanded, isUnused, isWatched, count, currencyType, icon, item = GetCurrencyListInfo( j )
		
		if not isHeader then
			
			if ArkInventory.db.char.option.ldb.tracking.currency.tracked[name] then
				
--				if currencyType == 1 then
--					icon = [[Interface\PVPFrame\PVP-ArenaPoints-Icon]]
--				elseif currencyType == 2 then
--					icon = [[Interface\PVPFrame\PVP-Currency-]] .. UnitFactionGroup( "player" )
--				end
				
				self.text = string.format( "%s  |T%s:0|t %d", self.text, icon or ArkInventory.Const.Texture.Missing, count or 0 )
				
			end
			
		end
		
	end
	
	self.text = string.trim( self.text )
	
end

function ArkInventory.LDB.Tracking_Currency.OnClick( frame, button )
	if button == "RightButton" then
		ArkInventory.MenuLDBTrackingCurrencyOpen( frame )
	else
		ArkInventory.Frame_Main_Toggle( ArkInventory.Const.Location.Token )
	end
end

function ArkInventory.LDB.Tracking_Currency:OnTooltipShow( )
	
	self:AddLine( string.format( "%s: %s", ArkInventory.Localise["LDB_TRACKING"], ArkInventory.Localise["LDB_TRACKING_CURRENCY"] ) )
	
	local numTokenTypes = GetCurrencyListSize( )
	
	if numTokenTypes == 0 then return end
	
	-- expand all token headers
	for j = numTokenTypes, 1, -1 do
		local _, isHeader, isExpanded = GetCurrencyListInfo( j )
		if isHeader and not isExpanded then
			ExpandCurrencyList( j, 1 )
		end
	end
	
	numTokenTypes = GetCurrencyListSize( )
	
	for j = 1, numTokenTypes do
		
		local name, isHeader, isExpanded, isUnused, isWatched, count, currencyType, icon, item = GetCurrencyListInfo( j )
		
		if isHeader then
			self:AddLine( " " )
			self:AddLine( name )
		else
			if ArkInventory.db.char.option.ldb.tracking.currency.tracked[name] then
				self:AddDoubleLine( name, count, 0, 1, 0, 0, 1, 0 )
			else
				self:AddDoubleLine( name, count, 1, 1, 1, 1, 1, 1 )
			end
		end
		
	end
	
end



function ArkInventory.LDB.Tracking_Item:Update( )
	
	if ArkInventory.IsCataclysm( ) then
		self:Update4( )
	else
		self:Update3( )
	end
	
end

function ArkInventory.LDB.Tracking_Item:Update3( )
	
	self.text = ""
	
	local numTracked = 0
	local size = ( ArkInventory.db.char.option.ldb.tracking.item.size or 0 ) + 1
	
	for k in ArkInventory.spairs( ArkInventory.db.global.option.tracking.items )  do
		
		if ArkInventory.db.char.option.ldb.tracking.item.tracked[k] then
			local count = GetItemCount( k )
			local icon = select( 10, GetItemInfo( k ) )
			numTracked = numTracked + 1
			size = size - 1
			self.text = string.format( "%s  |T%s:!!!:!!!|t %s", self.text, icon or ArkInventory.Const.Texture.Missing, count or 0 )
		end
		
	end
	
	if numTracked > 0 then
		if size < 1 then
			size = 0
		end
		size = string.format( "%d", size )
		self.text = string.gsub( self.text, "!!!", size )
	end
	
	self.text = string.trim( self.text )
	
end

function ArkInventory.LDB.Tracking_Item:Update4( )
	
	self.text = ""
	
	for k in ArkInventory.spairs( ArkInventory.db.global.option.tracking.items )  do
		if ArkInventory.db.char.option.ldb.tracking.item.tracked[k] then
			local count = GetItemCount( k )
			local icon = select( 10, GetItemInfo( k ) )
			self.text = string.format( "%s  |T%s:0|t %s", self.text, icon or ArkInventory.Const.Texture.Missing, count or 0 )
		end
	end
	
	self.text = string.trim( self.text )
	
end

function ArkInventory.LDB.Tracking_Item:OnClick( button )
	if button == "RightButton" then
		ArkInventory.MenuLDBTrackingItemOpen( self )
	end
end

function ArkInventory.LDB.Tracking_Item:OnTooltipShow( )
	
	self:AddLine( string.format( "%s: %s", ArkInventory.Localise["LDB_TRACKING"], ArkInventory.Localise["LDB_TRACKING_ITEM"] ) )
	
	self:AddLine( " " )
	
	for k in ArkInventory.spairs( ArkInventory.db.global.option.tracking.items )  do
		
		local count = GetItemCount( k )
		local name = GetItemInfo( k )
		
		local checked = ArkInventory.db.char.option.ldb.tracking.item.tracked[k]
		
		if checked then
			self:AddDoubleLine( name, count, 0, 1, 0, 0, 1, 0 )
		else
			self:AddDoubleLine( name, count, 1, 1, 1, 1, 1, 1 )
		end
		
	end
	
end



function ArkInventory.LDB.Pets:Update( )
	
	self.icon = [[Interface\Icons\INV_Misc_Dice_01]]
	
	local companionType = "CRITTER"
	
	local track = ArkInventory.db.char.option.ldb.pets.track
	
	if track then
		for k = 1, GetNumCompanions( companionType ) do
			local companionID, companionName, companionSpellID, texture, active = GetCompanionInfo( companionType, k )
			if companionSpellID == track then
				self.icon = texture
				return
			end
		end
		
	end
	
end

function ArkInventory.LDB.Pets:OnTooltipShow( )
	
	local companionType = "CRITTER"
	
	local n = GetNumCompanions( companionType )
	if n == 0 then
		self:AddLine( string.format( "%s: %s", ArkInventory.Global.Location[ArkInventory.Const.Location.Pet].Name, ArkInventory.Localise["LDB_COMPANION_NONE"] ) )
		return
	end
	
	local track = ArkInventory.db.char.option.ldb.pets.track
	
	if track then
		for k = 1, n do
			local companionID, companionName, companionSpellID, texture, active = GetCompanionInfo( companionType, k )
			if companionSpellID == track then
				self:AddLine( string.format( "%s: %s", ArkInventory.Global.Location[ArkInventory.Const.Location.Pet].Name, companionName ) )
				return
			end
		end
	else
		self:AddLine( string.format( "%s: %s", ArkInventory.Global.Location[ArkInventory.Const.Location.Pet].Name, ArkInventory.Localise["RANDOM"] ) )
	end

end

function ArkInventory.LDB.Pets:OnClick( button )

	if button == "RightButton" then
		
		ArkInventory.MenuLDBPetsOpen( self )
		
	else
		
		if UnitOnTaxi( "PLAYER" ) then return end

		local companionType = "CRITTER"
		
		if GetNumCompanions( companionType ) == 0 then return end
		
		local track = ArkInventory.db.char.option.ldb.pets.track
		
		if not track then
			
			-- random mount
			ArkInventory.LDB.Companion.GetAvailable( companionType )
			
			if #companionTable == 0 then return end
			
			CallCompanion( companionType, companionTable[random( 1, #companionTable )] )
			
		else
			
			-- specific pet
			
			local k = ArkInventory.LDB.Companion.GetSpecific( companionType, track )
			
			if k then
			
				if ArkInventory.IsCataclysm( ) then
					if IsUsableSpell( track ) then
						CallCompanion( companionType, k )
					else
						ArkInventory.OutputWarning( ArkInventory.Localise["LDB_COMPANION_SUMMON_FAIL"], string.lower( ArkInventory.Localise["WOW_ITEM_TYPE_MISC_PET"] ) )
						return ArkInventory.LDB.Mounts.OnClick( self, button, true )
					end
				else
					CallCompanion( companionType, k )
				end
					
			else
				
				ArkInventory.OutputError( ArkInventory.Localise["LDB_COMPANION_MISSING"] )
				ArkInventory.db.char.option.ldb.pets.track = nil
				
			end
			
		end
		
	end
	
end


function ArkInventory.LDB.Mounts:Update( )
	
	if ArkInventory.IsCataclysm( ) then
		self:Update4( )
	else
		self:Update3( )
	end
	
end

function ArkInventory.LDB.Mounts:Update3( )
	
	self.icon = [[Interface\Icons\INV_Misc_Dice_02]]
	
	local companionType = "MOUNT"
	
	local track
	
	if IsSwimming( ) then
		track = ArkInventory.db.char.option.ldb.mounts.water.track
	end
	
	if ArkInventory.LDB.Companion.IsFlyable( ) then
		track = ArkInventory.db.char.option.ldb.mounts.flying.track
	else
		track = ArkInventory.db.char.option.ldb.mounts.ground.track
	end
	
	if track then
		local k, _, _, _, texture = ArkInventory.LDB.Companion.GetSpecific( companionType, track )
		if k then
			self.icon = texture
		end
	end
	
end

function ArkInventory.LDB.Mounts:Update4( )
	
	--ArkInventory.Output( "Update4" )
	
	if InCombatLockdown( ) then return end
	
	local companionType = "MOUNT"
	
	self.icon = ArkInventory.Global.Location[ArkInventory.Const.Location.Mount].Texture
	
	local mode, track = ArkInventory.LDB.Mounts.GetPrimary( true )
	
	if mode then
		--local track = ArkInventory.db.char.option.ldb.mounts[mode].track
		if track == nil then
			self.icon = [[Interface\Icons\INV_Misc_Dice_02]]
		else
			local k, companionID, companionName, companionSpellID, texture, active = ArkInventory.LDB.Companion.GetSpecific( companionType, track )
			self.icon = texture
		end
	else
		--ArkInventory.Output( "none of your mounts are usable here" )
	end
	
end

function ArkInventory.LDB.Mounts:OnTooltipShow( ... )
	
	if ArkInventory.IsCataclysm( ) then
		ArkInventory.LDB.Mounts.OnTooltipShow4( self, ... )
	else
		ArkInventory.LDB.Mounts.OnTooltipShow3( self, ... )
	end
	
end

function ArkInventory.LDB.Mounts:OnTooltipShow3( ... )
	
	local companionType = "MOUNT"
	
	local n = GetNumCompanions( companionType )
	if n == 0 then
		self:AddLine( string.format( "%s: %s", ArkInventory.Global.Location[ArkInventory.Const.Location.Mount].Name, ArkInventory.Localise["LDB_COMPANION_NONE"] ) )
		return
	end
	
	local track = ArkInventory.db.char.option.ldb.mounts.ground.track
	if IsSwimming( ) then
		track = ArkInventory.db.char.option.ldb.mounts.water.track
	elseif ArkInventory.LDB.Companion.IsFlyable( ) then
		track = ArkInventory.db.char.option.ldb.mounts.flying.track
	end
	
	if track then
		local k, companionID, companionName, companionSpellID, texture, active = ArkInventory.LDB.Companion.GetSpecific( companionType, track )
		if k then
			self:AddLine( string.format( "%s: %s", ArkInventory.Global.Location[ArkInventory.Const.Location.Mount].Name, companionName ) )
			return
		end
	else
		self:AddLine( string.format( "%s: %s", ArkInventory.Global.Location[ArkInventory.Const.Location.Mount].Name, ArkInventory.Localise["RANDOM"] ) )
	end
	
end

function ArkInventory.LDB.Mounts:OnTooltipShow4( ... )
	
	if InCombatLockdown( ) then return end
	
	local companionType = "MOUNT"
	
	local n = GetNumCompanions( companionType )
	if n == 0 then
		self:AddLine( ArkInventory.Localise["LDB_COMPANION_NONE"] )
		return
	end
	
	local mode, track = ArkInventory.LDB.Mounts.GetPrimary( true )
	
	if mode then
		--local track = ArkInventory.db.char.option.ldb.mounts[mode].track
		if track == nil then
			mode = ArkInventory.Localise[string.upper( string.format( "LDB_MOUNTS_%s", mode ) )]
			self:AddLine( string.format( "%s: %s", mode, ArkInventory.Localise["RANDOM"] ) )
		else
			local k, companionID, companionName, companionSpellID, texture, active = ArkInventory.LDB.Companion.GetSpecific( companionType, track )
			self:AddLine( companionName )
		end
	else
		self:AddLine( ArkInventory.Localise["LDB_MOUNTS_NOT_HERE"] )
	end
	
end

function ArkInventory.LDB.Mounts:OnClick( ... )
	
	if ArkInventory.IsCataclysm( ) then
		ArkInventory.LDB.Mounts.OnClick4( self, ... )
	else
		ArkInventory.LDB.Mounts.OnClick3( self, ... )
	end
	
end

function ArkInventory.LDB.Mounts:OnClick3( button, forceRandom )
	
	--ArkInventory.Output( "OnClick3( ", button, ", ", forceRandom, " )" )
	
	if button == "RightButton" then
		
		ArkInventory.MenuLDBMountsOpen( self )
		
	else
		
		if ArkInventory.Global.Mode.Combat then return end
		if UnitOnTaxi( "PLAYER" ) then return end
		
		local companionType = "MOUNT"
		local mode = "ground"
		
		if GetNumCompanions( companionType ) == 0 then return end
		
		if IsMounted( ) then
			
			if IsFlying( ) then
				if not ArkInventory.db.char.option.ldb.mounts.flying.dismount then
					ArkInventory.OutputWarning( ArkInventory.Localise["LDB_MOUNTS_FLYING_DISMOUNT_WARNING"] )
				else
					Dismount( )
				end
			else
				Dismount( )
			end
			
			return
			
		end
		
		if IsSwimming( ) then
		
			--ArkInventory.Output( "water mount" )
			
			local track = ArkInventory.db.char.option.ldb.mounts.water.track
			
			if forceRandom or not track then
				
				--ArkInventory.Output( "random" )
				
				ArkInventory.LDB.Companion.GetAvailable( companionType, "water" )
				
				if #companionTable == 0 and ArkInventory.LDB.Companion.IsFlyable( ) then
					-- no valid water mounts, fallback to flying
					ArkInventory.LDB.Companion.GetAvailable( companionType, "flying" )
				end
				
				if #companionTable == 0 then
					-- no valid flying mounts, fallback to ground
					ArkInventory.LDB.Companion.GetAvailable( companionType, "ground" )
				end
				
				if #companionTable > 0 then
					CallCompanion( companionType, companionTable[random( 1, #companionTable )] )
				end
				
			else
				
				--ArkInventory.Output( "specific" )
				
				local k = ArkInventory.LDB.Companion.GetSpecific( companionType, track )
				
				if k then
					CallCompanion( companionType, k )
				else
					ArkInventory.OutputError( ArkInventory.Localise["LDB_COMPANION_MISSING"] )
					ArkInventory.db.char.option.ldb.mounts.water.track = nil
				end
				
			end
		
		elseif ArkInventory.LDB.Companion.IsFlyable( ) then
			
			--ArkInventory.Output( "flying mount" )
			
			local track = ArkInventory.db.char.option.ldb.mounts.flying.track
			
			if forceRandom or not track then
				
				--ArkInventory.Output( "random" )
				
				ArkInventory.LDB.Companion.GetAvailable( companionType, "flying" )
				
				if #companionTable == 0 then
					-- no valid flying mounts, fallback to ground mounts
					ArkInventory.LDB.Companion.GetAvailable( companionType, "ground" )
				end
				
				if #companionTable > 0 then	
					CallCompanion( companionType, companionTable[random( 1, #companionTable )] )
				end
				
			else
				
				--ArkInventory.Output( "specific" )
				
				local k = ArkInventory.LDB.Companion.GetSpecific( companionType, track )
				
				if k then
					CallCompanion( companionType, k )
				else
					ArkInventory.OutputError( ArkInventory.Localise["LDB_COMPANION_MISSING"] )
					ArkInventory.db.char.option.ldb.mounts.flying.track = nil
				end
				
			end
			
		else
			
			--ArkInventory.Output( "ground mount" )
			
			local track = ArkInventory.db.char.option.ldb.mounts.ground.track
			
			if forceRandom or not track then
				
				--ArkInventory.Output( "random" )
				
				ArkInventory.LDB.Companion.GetAvailable( companionType, "ground" )
				
				if #companionTable == 0 and ArkInventory.LDB.Companion.IsFlyable( ) then
					-- no valid water mounts, fallback to flying
					ArkInventory.LDB.Companion.GetAvailable( companionType, "flying" )
				end

				if #companionTable > 0 then
					CallCompanion( companionType, companionTable[random( 1, #companionTable )] )
				end
				
			else
				
				--ArkInventory.Output( "specific" )
				
				local k = ArkInventory.LDB.Companion.GetSpecific( companionType, track )
				
				if k then
					CallCompanion( companionType, k )
				else
					ArkInventory.OutputError( ArkInventory.Localise["LDB_COMPANION_MISSING"] )
					ArkInventory.db.char.option.ldb.mounts.ground.track = nil
				end
				
			end
			
		end
		
	end
	
end

function ArkInventory.LDB.Mounts:OnClick4( button, forceRandom )
	
	--ArkInventory.Output( "OnClick4( ", button, ", ", forceRandom, " )" )
	
	if InCombatLockdown( ) then return end
	
	if button == "RightButton" then
		
		ArkInventory.MenuLDBMountsOpen( self )
		
	else
		
		if UnitOnTaxi( "PLAYER" ) then return end
		
		local companionType = "MOUNT"
		
		if GetNumCompanions( companionType ) == 0 then return end
		
		ArkInventory.LDB.Mounts:Update( )
		
		if IsMounted( ) then
			
			if IsFlying( ) then
				if not ArkInventory.db.char.option.ldb.mounts.flying.dismount then
					ArkInventory.OutputWarning( ArkInventory.Localise["LDB_MOUNTS_FLYING_DISMOUNT_WARNING"] )
				else
					Dismount( )
				end
			else
				Dismount( )
			end
			
			return
			
		end
		
		
		local mode, track = ArkInventory.LDB.Mounts.GetPrimary( )
		
		if mode then
			--local track = ArkInventory.db.char.option.ldb.mounts[mode].track
			if track == nil then
				CallCompanion( companionType, companionTable[random( 1, #companionTable )] )
			else
				local k = ArkInventory.LDB.Companion.GetSpecific( companionType, track )
				CallCompanion( companionType, k )
			end
		else
			--ArkInventory.Output( "none of your mounts are usable here" )
		end
		
	end
	
end


function ArkInventory.LDB.Companion.IsFlyable( )

	local result = IsFlyableArea( )
	
	--ArkInventory.Output( result, " / ", GetRealZoneText( ), " / ", GetWintergraspWaitTime( ) )
	
	if result and GetRealZoneText( ) == ArkInventory.Localise["WOW_ZONE_WINTERGRASP"] and not GetWintergraspWaitTime( ) then
		--ArkInventory.Output( "battle in progress, no flying allowed" )
		result = false
	end
	
	--ArkInventory.Output( result, " / ", GetRealZoneText( ), " / ", " / ", GetWintergraspWaitTime( ) )
	
	return result
	
end

function ArkInventory.LDB.Companion.GetSpecific( companionType, track )
	
	local n = GetNumCompanions( companionType )
	for k = 1, n do
		local companionID, companionName, companionSpellID, texture, active = GetCompanionInfo( companionType, k )
		if companionSpellID == track then
			return k, companionID, companionName, companionSpellID, texture, active
		end
	end
	
end

function ArkInventory.LDB.Companion.GetAvailable( ... )
	
	if ArkInventory.IsCataclysm( ) then
		ArkInventory.LDB.Companion.GetAvailable4( ... )
	else
		ArkInventory.LDB.Companion.GetAvailable3( ... )
	end
	
end

function ArkInventory.LDB.Companion.GetAvailable3( companionType, mountType, ignoreActive )
	
	table.wipe( companionTable )
	
	local n = GetNumCompanions( companionType )
	
	local count = 0
	for creatureIndex = 1, n do
		
		local ok = true
		local companionID, companionName, companionSpellID, texture, active = GetCompanionInfo( companionType, creatureIndex )
		local companionData = ArkInventory.Const.CompanionData[companionSpellID]
		
		if companionData then
			
			if mountType then
				
				-- speed check for mounts
				
				ok = false
				
				local speed
				if mountType =="ground" then
					speed = companionData.sg
				elseif mountType =="flying" then
					speed = companionData.sf
				elseif mountType =="water" then
					speed = companionData.sw
				end
				
				if speed and ( speed == 0 or speed >= ArkInventory.db.char.option.ldb.mounts[mountType].min ) then
					ok = true
				--else
					--ArkInventory.Output( "ignoring ", companionLink, ", does not meet minimum speed or is not the requested type" )
				end
				
			end
			
			if ok and companionData.r then
				
				ok = false
				
				-- has a zone based restriction
				local zone = true
				if companionData.r.zone then
					zone = false
					--ArkInventory.Output( "zone [", companionData.r.zone, "] == [", GetRealZoneText( ), "]")
					for k in string.gmatch( companionData.r.zone, "[^,]+") do
						if GetRealZoneText( ) == ( ArkInventory.Localise[k] or "" ) then
							zone = true
							break
						end
					end
					--if not zone then
						--ArkInventory.Output( "ignoring ", companionLink, ", not in correct zone" )
					--end
				end
				
				-- requires an item based reagent
				local item = true
				if companionData.r.item then
					item = false
					if GetItemCount( companionData.r.item, false, true ) > 0 then
						item = true
					end
					--if not item then
						--ArkInventory.Output( "ignoring ", companionLink, ", no reagent found" )
					--end
				end
				
				if zone and item then
					ok = true
				else
					--ArkInventory.Output( "restricted ", creatureIndex, " / ", companionID, " ", companionLink )
				end
				
			end
			
			if ok then
				count = count + 1
				companionTable[count] = creatureIndex
			end
			
		end
		
	end
	
end

function ArkInventory.LDB.Companion.GetAvailable4( companionType, mountType, ignoreActive )
	
	table.wipe( companionTable )
	
	local n = GetNumCompanions( companionType )
	
	local count = 0
	for creatureIndex = 1, n do
		
		local ok = false
		local companionID, companionName, companionSpellID, texture, active = GetCompanionInfo( companionType, creatureIndex )
		
		if not active or ignoreActive then
			
			if IsUsableSpell( companionSpellID ) then
				
				--ArkInventory.Output( companionName, " is usable here" )
				
				if mountType then
					-- mounts
					local companionData = ArkInventory.Const.CompanionData[companionSpellID]
					if companionData then
						if mountType =="ground" and companionData.sg then
							ok = true
						elseif mountType =="flying" and companionData.sf then
							ok = true
						elseif mountType =="water" and companionData.sw then
							ok = true
						end
					end
				else
					--pets
					ok = true
				end
				
				if ok then
					count = count + 1
					companionTable[count] = creatureIndex
					--ArkInventory.Output( "usable: ", companionName )
				end
			
			end
			
		end
		
	end
	
end

function ArkInventory.LDB.Companion.HasActive( companionType )
	for creatureIndex = 1, GetNumCompanions( companionType ) do
		local companionID, companionName, companionSpellID, texture, active = GetCompanionInfo( companionType, creatureIndex )
		if active then
			return companionSpellID
		end
	end
end

function ArkInventory.LDB.Mounts.GetPrimary( ignoreActive )
	
	local companionType = "MOUNT"
	
	for _,v in pairs( ArkInventory.LDB.Mounts.types ) do
		
		--ArkInventory.Output( v, " mount" )
		
		local track = ArkInventory.db.char.option.ldb.mounts[v].track
		
		if track then
			-- check preferred mounts is usable
			--ArkInventory.Output( "check preferred" )
			local k, companionID, companionName, companionSpellID, texture, active = ArkInventory.LDB.Companion.GetSpecific( companionType, track )
			if k then
				if IsUsableSpell( companionSpellID ) then
					--ArkInventory.Output( "preferred ", v, " mount ", companionName, " is usable here" )
					return v, track
				else
					--ArkInventory.Output( "preferred ", v, " mount ", companionName, " is NOT usable here" )
				end
			else
				-- preferred mount is missing???, reset to random
				--ArkInventory.Output( "preferred ", v, " mount ", companionName, " is missing, resetting to random" )
				ArkInventory.db.char.option.ldb.mounts[v].track = nil
			end
		end
		
		if not track then
			-- preferred mount is not usable here or its set to random
			-- check how many mounts are usable here for this mode
			--ArkInventory.Output( "check random" )
			ArkInventory.LDB.Companion.GetAvailable( companionType, v, ignoreActive )
			if #companionTable > 0 then
				-- a random flying mount is usable here
				return v
			end
			--ArkInventory.Output( #companionTable, " ", v, " mounts are usable here" )
		end
		
	end
	
	--ArkInventory.Output( "none of your mounts are usable here" )
	
end
